/**
 *
 * Copyright (c) 2006-2015, Speedment, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); You may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at:
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.speedment.codegen.java.views;

import static com.speedment.codegen.util.Formatting.*;
import com.speedment.codegen.base.Generator;
import com.speedment.codegen.base.Transform;
import com.speedment.codegen.java.views.interfaces.HasAnnotationUsageView;
import com.speedment.codegen.java.views.interfaces.HasClassesView;
import com.speedment.codegen.java.views.interfaces.HasFieldsView;
import com.speedment.codegen.java.views.interfaces.HasJavadocView;
import com.speedment.codegen.java.views.interfaces.HasGenericsView;
import com.speedment.codegen.java.views.interfaces.HasInitalizersView;
import com.speedment.codegen.java.views.interfaces.HasImplementsView;
import com.speedment.codegen.java.views.interfaces.HasMethodsView;
import com.speedment.codegen.java.views.interfaces.HasModifiersView;
import com.speedment.codegen.java.views.interfaces.HasNameView;
import com.speedment.codegen.lang.models.ClassOrInterface;
import com.speedment.codegen.lang.models.Field;
import java.util.Optional;
import java.util.Collection;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 *
 * @author Emil Forslund
 * @param <M> The extending type
 */
public abstract class ClassOrInterfaceView<M extends ClassOrInterface<M>> implements 
    Transform<M, String>, HasNameView<M>, HasModifiersView<M>, HasJavadocView<M>, 
    HasGenericsView<M>, HasImplementsView<M>, HasInitalizersView<M>, HasMethodsView<M>,
    HasClassesView<M>, HasAnnotationUsageView<M>, HasFieldsView<M> {
    
	protected final static String
		CLASS_STRING = "class ",
		INTERFACE_STRING = "interface ",
		ENUM_STRING = "enum ",
		IMPLEMENTS_STRING = "implements ",
		EXTENDS_STRING = "extends ";

	protected String onBeforeFields(Generator cg, M model) {
		return EMPTY;
	}
	
	protected Object wrapField(Field field) {return field;}

    @Override
	public <In, C extends Collection<In>> Collection<Object> 
		wrap(C models, Function<In, Object> wrapper) {
		return models.stream().map(wrapper).collect(Collectors.toList());
	}

    @Override
    public String fieldSeparator() {
        return nl();
    }

    @Override
    public String fieldSuffix() {
        return SC;
    }
	
    protected abstract String renderDeclarationType();
	protected abstract String renderSupertype(Generator cg, M model);
    protected abstract String renderConstructors(Generator cg, M model);
    
	@Override
	public Optional<String> transform(Generator cg, M model) {
		return Optional.of(renderJavadoc(cg, model) +
            renderAnnotations(cg, model) +
			renderModifiers(cg, model) +
            renderDeclarationType() + 
            renderName(cg, model) + 
            renderGenerics(cg, model) +
            renderSupertype(cg, model) +
			renderInterfaces(cg, model) +
                
            // Code
            block(nl() + separate(
				onBeforeFields(cg, model), // Enums have constants here.
//				cg.onEach(wrap(model.getFields(), f -> wrapField(f)))
//					.collect(joinIfNotEmpty(scnl(), EMPTY, SC)),
                renderFields(cg, model),
				renderConstructors(cg, model),
                renderInitalizers(cg, model),
				renderMethods(cg, model),
				renderClasses(cg, model)
			))
		);
	}
	
	private String separate(Object... strings) {
		return Stream.of(strings)
			.map(o -> o.toString())
			.filter(s -> s.length() > 0)
			.collect(Collectors.joining(dnl()));
	}
}